/*
 * Copyright 2016 Ecole des Mines de Saint-Etienne.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.etsi.saref.server;

import com.github.mustachejava.DefaultMustacheFactory;
import com.github.mustachejava.MustacheFactory;
import com.github.mustachejava.MustacheResolver;
import com.github.mustachejava.resolver.DefaultResolver;
import java.util.HashMap;
import org.etsi.saref.server.entities.MediaTypeExt;
import java.util.List;
import java.util.Map;
import javax.ws.rs.core.Variant;
import org.apache.jena.shared.PrefixMapping;
import org.apache.jena.shared.impl.PrefixMappingImpl;
import org.apache.jena.sparql.expr.NodeValue;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.DefaultHandler;
import org.eclipse.jetty.server.handler.ErrorHandler;
import org.eclipse.jetty.server.handler.HandlerList;
import org.eclipse.jetty.server.handler.ResourceHandler;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.server.mvc.mustache.MustacheMvcFeature;
import org.glassfish.jersey.servlet.ServletContainer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Maxime Lefrançois <maxime.lefrancois at emse.fr>
 */
public class SAREFServer{

    public static final String NS = "https://saref.etsi.org/";
    public static final String DS = "http://localhost:3030/ds/";
    public static final String CONFIG = "http://example.org/" + "config";

    private static final Logger LOG = LoggerFactory.getLogger(SAREFServer.class);

    public static final MediaTypeExt TEXT_HTML = new MediaTypeExt("text", "html", "html", null, false);
    public static final MediaTypeExt APPLICATION_RDFXML = new MediaTypeExt("application", "rdf+xml", "rdf", "RDF/XML", true);
    public static final MediaTypeExt TEXT_TURTLE = new MediaTypeExt("text", "turtle", "ttl", "TTL", true);
    public static final MediaTypeExt APPLICATION_NTRIPLE = new MediaTypeExt("application", "n-triples", "nt", "N-TRIPLE", true);
    public static final MediaTypeExt APPLICATION_N3 = new MediaTypeExt("application", "n3", "n3", "N3", true);
    public static final MediaTypeExt APPLICATION_JSONLD = new MediaTypeExt("application", "ld+json", "jsonld", "JSON-LD", true);
    public static final MediaTypeExt APPLICATION_XHTMLXML = new MediaTypeExt("application", "xhtml+xml", "html", null, false);

    public static final List<Variant> VARIANTS = Variant.mediaTypes(
            TEXT_HTML,
            APPLICATION_RDFXML,
            TEXT_TURTLE,
            APPLICATION_NTRIPLE,
            APPLICATION_N3,
            APPLICATION_JSONLD,
            APPLICATION_XHTMLXML).build();

    public static final Map<String, MediaTypeExt> EXTENSIONS;

    static {
        EXTENSIONS = new HashMap<>();
        for (Variant variant : VARIANTS) {
            MediaTypeExt mt = (MediaTypeExt) variant.getMediaType();
            EXTENSIONS.put(mt.getExt(), mt);
        }
    }

    public static final PrefixMapping PM = new PrefixMappingImpl();

    static {
        PM.setNsPrefix("owl", "http://www.w3.org/2002/07/owl#");
        PM.setNsPrefix("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
        PM.setNsPrefix("xml", "http://www.w3.org/XML/1998/namespace");
        PM.setNsPrefix("xsd", "http://www.w3.org/2001/XMLSchema#");
        PM.setNsPrefix("foaf", "http://xmlns.com/foaf/0.1/");
        PM.setNsPrefix("rdfs", "http://www.w3.org/2000/01/rdf-schema#");
        PM.setNsPrefix("vann", "http://purl.org/vocab/vann/");
        PM.setNsPrefix("voaf", "http://purl.org/vocommons/voaf#");
        PM.setNsPrefix("dcterms", "http://purl.org/dc/terms/");
        PM.setNsPrefix("saref", "https://saref.etsi.org/core/");
    }

    public static void main(String[] args) {
        NodeValue.VerboseExceptions = true;
        NodeValue.VerboseWarnings = false;

        ResourceConfig resourceConfig = new ResourceConfig();
        resourceConfig.property(MustacheMvcFeature.TEMPLATE_BASE_PATH, "/templates");
        MustacheResolver resolver = new DefaultResolver(".");
        MustacheFactory factory = new DefaultMustacheFactory(resolver);
        resourceConfig.property(MustacheMvcFeature.TEMPLATE_OBJECT_FACTORY, factory);
        resourceConfig.register(MustacheMvcFeature.class);
        resourceConfig.packages("org.etsi.saref.server.resources");
        ServletHolder servletHolder = new ServletHolder(new ServletContainer(resourceConfig));

        ServletContextHandler contextHandler = new ServletContextHandler();
        contextHandler.addServlet(servletHolder, "/*");
        
        ResourceHandler resourceHandler = new ResourceHandler();
        resourceHandler.setDirectoriesListed(true);
        resourceHandler.setResourceBase("src/main/webapp");

        HandlerList handlers = new HandlerList();
        handlers.setHandlers(new Handler[]{
            resourceHandler,
            contextHandler,
            new ErrorHandler(),
            new DefaultHandler()});
        
        Server server = new Server(8080);
        server.setHandler(handlers);

        try {
            server.start();
            server.join();
        } catch (Exception ex) {
            try {
                server.stop();
            } catch (Exception e) {
                System.exit(-1);
            }
        }
    }

}
